import { type HanziData, type TradOverrides } from './hanzi-data'
import cnchar from 'cnchar'
import cncharOrder from 'cnchar-order'
import cncharRadical from 'cnchar-radical'
import cncharTrad from 'cnchar-trad'
import { getTraditionalInfo } from './traditional'
import { type HanziWriterData, loadHanziWriterData } from './hanzi-writer'

cnchar.use(cncharOrder, cncharRadical, cncharTrad)

export interface QueryOpts {
  readonly char: string
}

const noHanziWriterData: HanziWriterData = { strokes: [], medians: [] }

export async function query ({ char }: QueryOpts): Promise<HanziData> {
  if (char.length !== 1) {
    throw new Error(`Can only query single chars, got: ${char}`)
  }
  const data = await queryDefault(char)
  data.trad = await queryTraditional(char, data)
  return data
}

async function queryDefault (char: string): Promise<HanziData> {
  const hanziWriterData = (await loadHanziWriterData(char)) ??
    noHanziWriterData
  const totalStrokeCount = hanziWriterData.strokes.length
  const simplified = cnchar.radical(char)[0]
  const radical = simplified.radical
  const extra = Math.max(0, totalStrokeCount - simplified.radicalCount)
  const count = `${simplified.radicalCount}+${extra}`
  const strokeShapes = cnchar.stroke(char, 'order', 'shape')[0]
  const strokeTypes = (
    Array.isArray(strokeShapes) &&
    strokeShapes.length === hanziWriterData.strokes.length
  )
  // if cnchar has multiple stroke types, only keep first
    ? strokeShapes.map(t => t.split('|')[0])
    : []
  if (strokeTypes.length === 0) {
    console.error(`stroke table data unavailable for ${char}`)
  }
  return {
    radical,
    count,
    strokeTypes,
    ...hanziWriterData,
    trad: {}
  }
}

async function queryTraditional (
  char: string,
  defaultData: HanziData
): Promise<TradOverrides> {
  const overrides: TradOverrides = {}
  const traditional = await getTraditionalInfo(char)
  for (const prop of ['radical', 'count']) {
    if (traditional[prop] !== defaultData[prop]) {
      overrides[prop] = traditional[prop]
    }
  }
  return overrides
}
